import os
import gdal
import numpy as np

def readTif(fileName):
    dataset = gdal.Open(fileName)
    if dataset == None:
        print(fileName + "The file could not be opened")
    return dataset


# Save the TIF file function
def writeTiff(im_data, im_geotrans, im_proj, path):
    if 'int8' in im_data.dtype.name:
        datatype = gdal.GDT_Byte
    elif 'int16' in im_data.dtype.name:
        datatype = gdal.GDT_UInt16
    else:
        datatype = gdal.GDT_Float32
    if len(im_data.shape) == 3:
        im_bands, im_height, im_width = im_data.shape
    elif len(im_data.shape) == 2:
        im_data = np.array([im_data])
        im_bands, im_height, im_width = im_data.shape
    # Create a file
    driver = gdal.GetDriverByName("GTiff")
    dataset = driver.Create(path, int(im_width), int(im_height), int(im_bands), datatype)
    print(dataset)
    if (dataset != None):
        dataset.SetGeoTransform(im_geotrans)  # Writes an affine transform parameter
        dataset.SetProjection(im_proj)  # Write the projection
    for i in range(im_bands):
        dataset.GetRasterBand(i + 1).WriteArray(im_data[i])
    del dataset

def MonthlyAverage(tifFolder, SaveFolder, suffix):
    '''
    The folder where the TIF is located
    suffix Generates the result filename suffix
    '''
    #  Gets the file name in the folder
    tifNameList = os.listdir(tifFolder)
    #  Gets the month of the file
    YearMonthList = []
    for i in range(len(tifNameList)):
        #  My top seven filenames are year-month, such as 2012-06
        YearMonth = tifNameList[i][0:7]
        YearMonthList.append(YearMonth)
    #  Removes duplicates
    YearMonthList = list(set(YearMonthList))
    #  Match the corresponding files for each month to average
    for i in range(len(YearMonthList)):
        tifPath = tifFolder + "/" + tifNameList[0]
        dataset = readTif(tifPath)
        width = dataset.RasterXSize #The number of columns in the raster matrix
        height = dataset.RasterYSize #The number of rows in the raster matrix
        Tif_geotrans = dataset.GetGeoTransform()
        Tif_proj = dataset.GetProjection()
        data_sum = np.zeros((height, width))
        count = 0
        for j in range(len(tifNameList)):
            if(tifNameList[j][0:7] == YearMonthList[i]):
                tifPath = tifFolder + "/" + tifNameList[j]
                dataset = readTif(tifPath)
                Tif_data = dataset.ReadAsArray(0,0,width,height)#Gets the data
                data_sum = data_sum + Tif_data
                count = count + 1
        MonthlyAverage = data_sum / count
        SavePath = SaveFolder + "/" + YearMonthList[i] + suffix + ".tif";
        writeTiff(MonthlyAverage,Tif_geotrans,Tif_proj,SavePath)
        print(SavePath, " Monthly data generation successful！")

MonthlyAverage(r"C:\Users\Administrator\Desktop\data\modis\MOD11A2\05LST-Correct",
               r"C:\Users\Administrator\Desktop\data\modis\MOD11A2\LST_mean_merge", "Month")