import gdal, osr
import numpy as np
import os
import math

# Read the TIF data set
def readTif(fileName):
    dataset = gdal.Open(fileName)
    if dataset == None:
        print(fileName + "The file could not be opened")
    return dataset


# Save the TIF file function
def writeTiff(im_data, im_geotrans, im_proj, path):
    if 'int8' in im_data.dtype.name:
        datatype = gdal.GDT_Byte
    elif 'int16' in im_data.dtype.name:
        datatype = gdal.GDT_UInt16
    else:
        datatype = gdal.GDT_Float32
    if len(im_data.shape) == 3:
        im_bands, im_height, im_width = im_data.shape
    elif len(im_data.shape) == 2:
        im_data = np.array([im_data])
        im_bands, im_height, im_width = im_data.shape
    # Create a file
    driver = gdal.GetDriverByName("GTiff")
    dataset = driver.Create(path, int(im_width), int(im_height), int(im_bands), datatype)
    print(dataset)
    if (dataset != None):
        dataset.SetGeoTransform(im_geotrans)  # Writes an affine transform parameter
        dataset.SetProjection(im_proj)  # Write the projection
    for i in range(im_bands):
        dataset.GetRasterBand(i + 1).WriteArray(im_data[i])
    del dataset


#  Batch average interpolation
def SpatialInterpolation_batch(tifFolder, FillValue, SaveFolder, suffix, ScaleFactor):
    #  Gets the file name in the folder
    tifNameList = os.listdir(tifFolder)
    for i in range(len(tifNameList)):
        #  Determines whether the current file is a TIF file
        if (os.path.splitext(tifNameList[i])[1] == ".tif"):
            tifPath = tifFolder + "/" + tifNameList[i]
            dataset = readTif(tifPath)
            width = dataset.RasterXSize  # The number of columns in the raster matrix
            height = dataset.RasterYSize  # The number of rows in the raster matrix
            Tif_geotrans = dataset.GetGeoTransform()
            Tif_proj = dataset.GetProjection()
            Tif_data = dataset.ReadAsArray(0, 0, width, height)  # Gets the data

            mean = np.mean(Tif_data)
            for j in range(Tif_data.shape[0]):
                for k in range(Tif_data.shape[1]):
                    if (math.isclose(Tif_data[j][k], FillValue)):
                        Tif_data[j][k] = mean

            Tif_data = Tif_data * ScaleFactor
            Savepath = SaveFolder + "/" + os.path.splitext(tifNameList[i])[0] + suffix + ".tif"
            writeTiff(Tif_data, Tif_geotrans, Tif_proj, Savepath)
            print(Savepath + " Interpolation successful!")


# #  NDVI Interpolation
# SpatialInterpolation_batch(r"C:\Users\Administrator\Desktop\data\modis\MOD11A3\NDVI-TIF-WARP", -3000,
#                            r"C:\Users\Administrator\Desktop\data\modis\MOD11A3\NDVI-SpatialInterpolation", "_SpatialIinterpolation", 0.0001)
# # EVI Interpolation
# SpatialInterpolation_batch(r"C:\Users\Administrator\Desktop\data\modis\MOD11A3\EVI-TIF-WARP", -3000,
#                            r"C:\Users\Administrator\Desktop\data\modis\MOD11A3\EVI-SpatialInterpolation", "_SpatialIinterpolation", 0.0001)
#  LST Interpolation
SpatialInterpolation_batch(r"C:\Users\Administrator\Desktop\data\modis\MOD11A2\03LST-TIF-WARP\1", 0,
                           r"C:\Users\Administrator\Desktop\data\modis\MOD11A2\03LST-TIF-WARP\1\00", "_interpolation", 0.02)