#coding=utf-8
import gdal, osr
import numpy as np
import os
import math

#S_G_filter
from scipy.signal import savgol_filter

def readTif(fileName):
    dataset = gdal.Open(fileName)
    if dataset == None:
        print(fileName + "The file could not be opened")
    return dataset


# Save the TIF file function
def writeTiff(im_data, im_geotrans, im_proj, path):
    if 'int8' in im_data.dtype.name:
        datatype = gdal.GDT_Byte
    elif 'int16' in im_data.dtype.name:
        datatype = gdal.GDT_UInt16
    else:
        datatype = gdal.GDT_Float32
    if len(im_data.shape) == 3:
        im_bands, im_height, im_width = im_data.shape
    elif len(im_data.shape) == 2:
        im_data = np.array([im_data])
        im_bands, im_height, im_width = im_data.shape
    # 创建文件
    driver = gdal.GetDriverByName("GTiff")
    dataset = driver.Create(path, int(im_width), int(im_height), int(im_bands), datatype)
    print(dataset)
    if (dataset != None):
        dataset.SetGeoTransform(im_geotrans)  # Writes an affine transform parameter
        dataset.SetProjection(im_proj)  # Write the projection
    for i in range(im_bands):
        dataset.GetRasterBand(i + 1).WriteArray(im_data[i])
    del dataset

def SG_filter(tifFolder, SaveFolder, suffix):
    '''
    The folder where the TIF is located
    suffix Generates the result filename suffix
    '''
    #  Gets the file name in the folder
    tifNameList = os.listdir(tifFolder)
    tifPath = tifFolder + "/" + tifNameList[0]
    dataset = readTif(tifPath)
    width = dataset.RasterXSize  # The number of columns in the raster matrix
    height = dataset.RasterYSize  # The number of rows in the raster matrix
    Tif_geotrans = dataset.GetGeoTransform()
    Tif_proj = dataset.GetProjection()
    Tif_data = dataset.ReadAsArray(0, 0, width, height)  # Gets the data
    Tif_datas = np.zeros((len(tifNameList), Tif_data.shape[0], Tif_data.shape[1]))
    for i in range(len(tifNameList)):
        tifPath = tifFolder + "/" + tifNameList[i]
        dataset = readTif(tifPath)
        Tif_data = dataset.ReadAsArray(0, 0, width, height)  # Gets the data
        Tif_datas[i] = Tif_data
    #  Dimension switching to facilitate subsequent extraction of each pixel corresponding to the respective period values
    Tif_datas = Tif_datas.swapaxes(1, 0)
    Tif_datas = Tif_datas.swapaxes(1, 2)

    SGfilter = np.zeros(Tif_datas.shape)
    #  Calculate the SG filter for each grid position
    for i in range(Tif_datas.shape[0]):
        for j in range(Tif_datas.shape[1]):
            SGfilter[i][j] = savgol_filter(Tif_datas[i][j], window_length=7, polyorder=2)
            #  The dimensions switch back
    SGfilter = SGfilter.swapaxes(1, 0)
    SGfilter = SGfilter.swapaxes(0, 2)

    for i in range(SGfilter.shape[0]):
        SavePath = SaveFolder + "/" + os.path.splitext(tifNameList[i])[0] + suffix + ".tif"
        writeTiff(SGfilter[i], Tif_geotrans, Tif_proj, SavePath)
        print(SavePath + " S-G filter successful!")


# #  The NDVI is SG filtered
# SG_filter(r"C:\Users\Administrator\Desktop\data\modis\MOD11A3\NDVI-SpatialInterpolation\132",
#           r"C:\Users\Administrator\Desktop\data\modis\MOD11A3\NDVI-SGFilter", "_SGfilter")
#  The EVI is SG-filtered
# SG_filter(r"C:\Users\Administrator\Desktop\data\modis\MOD11A3\EVI-SpatialInterpolation\100",
#           r"C:\Users\Administrator\Desktop\data\modis\MOD11A3\EVI-SGFilter", "_SGfilter")
#  The LST is SG filtered
SG_filter(r"C:\Users\Administrator\Desktop\data\modis\MOD11A2\06resample\132",
          r"C:\Users\Administrator\Desktop\data\modis\MOD11A2\07SG-filter", "_SGfilter")
