import numpy as np
import os
import gdal, osr


def readTif(fileName):
    dataset = gdal.Open(fileName)
    if dataset == None:
        print(fileName + "The file could not be opened")
    return dataset


# Save the TIF file function
def writeTiff(im_data, im_geotrans, im_proj, path):
    if 'int8' in im_data.dtype.name:
        datatype = gdal.GDT_Byte
    elif 'int16' in im_data.dtype.name:
        datatype = gdal.GDT_UInt16
    else:
        datatype = gdal.GDT_Float32
    if len(im_data.shape) == 3:
        im_bands, im_height, im_width = im_data.shape
    elif len(im_data.shape) == 2:
        im_data = np.array([im_data])
        im_bands, im_height, im_width = im_data.shape
    # Create a file
    driver = gdal.GetDriverByName("GTiff")
    dataset = driver.Create(path, int(im_width), int(im_height), int(im_bands), datatype)
    print(dataset)
    if (dataset != None):
        dataset.SetGeoTransform(im_geotrans)  # Writes an affine transform parameter
        dataset.SetProjection(im_proj)  # Write the projection
    for i in range(im_bands):
        dataset.GetRasterBand(i + 1).WriteArray(im_data[i])
    del dataset

def LSTCorrection(LSTFolder, DEMPath, SaveFloder, suffix, a, b, c):
    dataset = readTif(DEMPath)
    width = dataset.RasterXSize  # The number of columns in the raster matrix
    height = dataset.RasterYSize  # The number of rows in the raster matrix
    DEM_geotrans = dataset.GetGeoTransform()
    DEM_proj = dataset.GetProjection()
    DEM_data = dataset.ReadAsArray(0, 0, width, height)  # Gets the data
    #  Gets the file name in the folder
    LSTNameList = os.listdir(LSTFolder)
    for i in range(len(LSTNameList)):
        LSTPath = LSTFolder + "/" + LSTNameList[i]
        dataset = readTif(LSTPath)
        LST_data = dataset.ReadAsArray(0, 0, width, height)  # Gets the data
        LST_data[LST_data < 0] = 0
        print(LST_data)
        LSTCorr = np.zeros(LST_data.shape)

        for j in range(LST_data.shape[0]):
            for k in range(LST_data.shape[1]):
                #  Calculate latitude from affine transformation matrix
                Latitude = DEM_geotrans[3] + DEM_geotrans[4] * j + DEM_geotrans[5] * k
                #  LST is corrected according to the formula
                LSTCorr[j][k] = LST_data[j][k] + a * DEM_data[j][k] + b * Latitude + c
        SavePath = SaveFloder + "/" + os.path.splitext(LSTNameList[i])[0] + suffix + ".tif"
        writeTiff(LSTCorr, DEM_geotrans, DEM_proj, SavePath)
        print(SavePath + "Calibration successful！")


LSTCorrection(r"C:\Users\Administrator\Desktop\data\modis\MOD11A2\07SG-filter",
              r"C:\Users\Administrator\Desktop\data\modis\DEM\DEM_warp.tif",
              r"C:\Users\Administrator\Desktop\data\modis\MOD11A2\08LST-Correct-new",
              "_Correction", 0.006, 0.4, -16)