import os
import gdal, osr
import numpy as np

#  HDF batch to TIF

import os

from main.hdfToTif import array2raster


def hdf2tif_batch(hdfFolder, SaveFolder, choice):
    #  Gets the file name in the folder
    hdfNameList = os.listdir(hdfFolder)
    for i in range(len(hdfNameList)):
        #  Determines whether the current file is an HDF file
        if (os.path.splitext(hdfNameList[i])[1] == ".hdf"):
            hdfPath = hdfFolder + "/" + hdfNameList[i]
            #  GDAL opens the HDF data set
            datasets = gdal.Open(hdfPath)

            #  Extract the first child data set, the LST data
            DatasetLST = datasets.GetSubDatasets()[0][0]
            RasterLST = gdal.Open(DatasetLST)
            LST = RasterLST.ReadAsArray()

            #  Extract the second subdata set, the QC data
            DatasetQC = datasets.GetSubDatasets()[1][0]
            RasterQC = gdal.Open(DatasetQC)
            QC = RasterQC.ReadAsArray()

            # Convert decimal to binary
            binary_repr_v = np.vectorize(np.binary_repr)
            QC_binary = binary_repr_v(QC, 8)

            # 6-7 bits, is the field that controls the quality of the LST, and &#039;00&#039; stands for LST Error Flag & Lt; = 1K
            #  Np. Char. Count is a function that counts the number of occurrences of substrings in a string. We use this function to determine whether a particular position of a string is consistent with a substring
            #  The 6-7 bit position in Python corresponds to the 0-1 position
            Error_mask = np.char.count(QC_binary, '00', start=0, end=2) == 1
            #  Extract the ones that satisfy the quality condition, and set the ones that don&#039;t to FillValue 0
            LST_New = np.where(Error_mask, LST, 0)

            #  Gets the metadata in the HDF
            Metadata = datasets.GetMetadata()
            #  Gets the dimensions of the four corners
            Latitudes = Metadata["GRINGPOINTLATITUDE.1"]
            #  Use”,” for segmentation
            LatitudesList = Latitudes.split(", ")
            #  Gets the longitude of the four corners
            Longitude = Metadata["GRINGPOINTLONGITUDE.1"]
            #  Use”,” for segmentation
            LongitudeList = Longitude.split(", ")

            # The geographic coordinates of the four corners of the image
            GeoCoordinates = np.zeros((4, 2), dtype="float32")
            GeoCoordinates[0] = np.array([float(LongitudeList[0]), float(LatitudesList[0])])
            GeoCoordinates[1] = np.array([float(LongitudeList[1]), float(LatitudesList[1])])
            GeoCoordinates[2] = np.array([float(LongitudeList[2]), float(LatitudesList[2])])
            GeoCoordinates[3] = np.array([float(LongitudeList[3]), float(LatitudesList[3])])

            #  Column number
            Columns = LST_New.shape[0]
            #  Number of rows
            Rows = LST_New.shape[1]
            #  Image coordinates for the four corners of the image
            PixelCoordinates = np.array([[0, 0],
                                         [Columns - 1, 0],
                                         [Columns - 1, Rows - 1],
                                         [0, Rows - 1]], dtype="float32")

            #  The affine transformation matrix is calculated
            from scipy.optimize import leastsq
            def func(i):
                Transform0, Transform1, Transform2, Transform3, Transform4, Transform5 = i[0], i[1], i[2], i[3], i[4], \
                                                                                         i[5]
                return [Transform0 + PixelCoordinates[0][0] * Transform1 + PixelCoordinates[0][1] * Transform2 -
                        GeoCoordinates[0][0],
                        Transform3 + PixelCoordinates[0][0] * Transform4 + PixelCoordinates[0][1] * Transform5 -
                        GeoCoordinates[0][1],
                        Transform0 + PixelCoordinates[1][0] * Transform1 + PixelCoordinates[1][1] * Transform2 -
                        GeoCoordinates[1][0],
                        Transform3 + PixelCoordinates[1][0] * Transform4 + PixelCoordinates[1][1] * Transform5 -
                        GeoCoordinates[1][1],
                        Transform0 + PixelCoordinates[2][0] * Transform1 + PixelCoordinates[2][1] * Transform2 -
                        GeoCoordinates[2][0],
                        Transform3 + PixelCoordinates[2][0] * Transform4 + PixelCoordinates[2][1] * Transform5 -
                        GeoCoordinates[2][1],
                        Transform0 + PixelCoordinates[3][0] * Transform1 + PixelCoordinates[3][1] * Transform2 -
                        GeoCoordinates[3][0],
                        Transform3 + PixelCoordinates[3][0] * Transform4 + PixelCoordinates[3][1] * Transform5 -
                        GeoCoordinates[3][1]]

            #  Least squares
            GeoTransform = leastsq(func, np.asarray((1, 1, 1, 1, 1, 1)))

            #  Gets the data time
            date = Metadata["RANGEBEGINNINGDATE"]

            TifName = SaveFolder + "/" + date + ".tif"
            array2raster(TifName, GeoTransform[0], LST_New)
            print(TifName, "Conversion successful！")


hdf2tif_batch(r"C:\Users\Administrator\Desktop\data\modis\MOD11A2\00original-data", r"C:\Users\Administrator\Desktop\data\modis\MOD11A2\01LST-TIF",0)
