import gdal, osr
import numpy as np

# 1.GDAL opens the HDF data set
datasets = gdal.Open(r"C:\Users\Administrator\Desktop\data\modis\MOD11A2\00original-data\MOD11A2.A2018001.h28v06.006.2018011144414.hdf")
#
 # Gets the child data set in the HDF
SubDatasets = datasets.GetSubDatasets()
#  Gets the number of child datasets
SubDatasetsNum = len(datasets.GetSubDatasets())
#  Outputs information for each subdataset
print("The child data sets are common{0}: ".format(SubDatasetsNum))
for i in range(SubDatasetsNum):
    print(datasets.GetSubDatasets()[i])

#
#
# #2. Gets the metadata in the HDF
Metadata = datasets.GetMetadata()
#  Gets the number of metadata
MetadataNum = len(Metadata)
#  Outputs information for each subdataset
print("Metadata is common{0}: ".format(MetadataNum))
for key, value in Metadata.items():
    print('{key}:{value}'.format(key=key, value=value))

#  Extract the first child data set, the NDVI data
print("----------------------------")
print(datasets.GetSubDatasets()[1])
print("----------------------------")


DatasetNDVI = datasets.GetSubDatasets()[0][0]
RasterNDVI = gdal.Open(DatasetNDVI)
NDVI = RasterNDVI.ReadAsArray()
#
# #  Gets the latitude of the four corners
Latitudes = Metadata["GRINGPOINTLATITUDE.1"]
#  Use”,” for segmentation
LatitudesList = Latitudes.split(", ")
#  Gets the longitude of the four corners
Longitude = Metadata["GRINGPOINTLONGITUDE.1"]
#  Use”,” for segmentation
LongitudeList = Longitude.split(", ")

# The geographic coordinates of the four corners of the image
GeoCoordinates = np.zeros((4, 2), dtype="float32")
GeoCoordinates[0] = np.array([float(LongitudeList[0]), float(LatitudesList[0])])
GeoCoordinates[1] = np.array([float(LongitudeList[1]), float(LatitudesList[1])])
GeoCoordinates[2] = np.array([float(LongitudeList[2]), float(LatitudesList[2])])
GeoCoordinates[3] = np.array([float(LongitudeList[3]), float(LatitudesList[3])])

Columns = NDVI.shape[0]
Rows = NDVI.shape[1]
PixelCoordinates = np.array([[0, 0],
                             [Columns - 1, 0],
                             [Columns - 1, Rows - 1],
                             [0, Rows - 1]], dtype="float32")

#  The affine transformation matrix is calculated
from scipy.optimize import leastsq
#
#
def func(i):
    Transform0, Transform1, Transform2, Transform3, Transform4, Transform5 = i[0], i[1], i[2], i[3], i[4], i[5]
    return [
        Transform0 + PixelCoordinates[0][0] * Transform1 + PixelCoordinates[0][1] * Transform2 - GeoCoordinates[0][0],
        Transform3 + PixelCoordinates[0][0] * Transform4 + PixelCoordinates[0][1] * Transform5 - GeoCoordinates[0][1],
        Transform0 + PixelCoordinates[1][0] * Transform1 + PixelCoordinates[1][1] * Transform2 - GeoCoordinates[1][0],
        Transform3 + PixelCoordinates[1][0] * Transform4 + PixelCoordinates[1][1] * Transform5 - GeoCoordinates[1][1],
        Transform0 + PixelCoordinates[2][0] * Transform1 + PixelCoordinates[2][1] * Transform2 - GeoCoordinates[2][0],
        Transform3 + PixelCoordinates[2][0] * Transform4 + PixelCoordinates[2][1] * Transform5 - GeoCoordinates[2][1],
        Transform0 + PixelCoordinates[3][0] * Transform1 + PixelCoordinates[3][1] * Transform2 - GeoCoordinates[3][0],
        Transform3 + PixelCoordinates[3][0] * Transform4 + PixelCoordinates[3][1] * Transform5 - GeoCoordinates[3][1]]


#  Least squares
GeoTransform = leastsq(func, np.asarray((1, 1, 1, 1, 1, 1)))
print("Affine transformation matrix: ", GeoTransform)


# #  Writes the TIF function
def array2raster(TifName, GeoTransform, array):
    cols = array.shape[1]  # The number of matrix columns
    rows = array.shape[0]  # The number of matrix rows
    driver = gdal.GetDriverByName('GTiff')
    outRaster = driver.Create(TifName, cols, rows, 1, gdal.GDT_Float32)
    # The two zeros in parentheses indicate that the row and column numbers of the starting pixel begin with (0,0)
    outRaster.SetGeoTransform(tuple(GeoTransform))
    # Get the first band of the dataset, starting at 1, not 0
    outband = outRaster.GetRasterBand(1)
    outband.WriteArray(array)
    outRasterSRS = osr.SpatialReference()
    # The code 4326 represents the WGS84 coordinates
    outRasterSRS.ImportFromEPSG(4326)
    outRaster.SetProjection(outRasterSRS.ExportToWkt())
    outband.FlushCache()

#
#  Gets the data time as the output file name
date = Metadata["RANGEBEGINNINGDATE"]
TifName = date + ".tif"
array2raster(TifName, GeoTransform[0], NDVI)
print("HDF to TIF successful！")
